import 'package:flutter/material.dart';
import 'package:getx_extended/export.dart';

class RxPageable<T> extends RxList<T> {
  static const PAGE_SIZE = 20;
  static const DEFAULT_FIRST_PAGE = 1;

  Future<List<T>> Function(int page, int size) dataSource;

  RxPageable({
    required this.dataSource,
  }) : super([]) {
    clear();
  }

  late int _currentPage;
  late bool _isEnded;
  late bool _isLoading;

  Future clear() async {
    assignAll([]);
    _currentPage = DEFAULT_FIRST_PAGE;
    _isEnded = false;
    _isLoading = false;
    await next();
  }

  T item(int index) {
    if (index + 6 > value.length) {
      next();
    }
    return value[index];
  }

  Future next() async {
    if (_isEnded || _isLoading) return;
    try {
      _isLoading = true;
      final list = await dataSource(_currentPage, PAGE_SIZE);

      WidgetsBinding.instance?.addPostFrameCallback((timeStamp) {
        addAllIf(true, list);
      });

      _currentPage += 1;
      _isEnded = PAGE_SIZE > list.length;
    } finally {
      _isLoading = false;
    }
  }
}
