import 'package:flutter/material.dart';
import 'package:getx_extended/export.dart';

class AutoFetchList<T> extends StatelessWidget {
  Widget Function(T item) builder;
  RxPageable<T> source;
  bool reverse;

  Widget? header;
  Widget? separator;
  Widget? empty;
  EdgeInsetsGeometry? padding;
  VoidCallback? next;

  AutoFetchList({
    required this.builder,
    required this.source,
    this.reverse = false,
    this.header,
    this.separator,
    this.next,
    this.padding,
    this.empty,
  });

  @override
  Widget build(BuildContext context) => ObxValue<RxList<T>>(
        (rx) => Stack(
          children: [
            _body(rx),
            _empty(rx, empty),
          ],
        ),
        source,
      );

  Widget _body(RxList<T> rx) {
    final offset = header != null ? 1 : 0;
    return ListView.separated(
      padding: padding,
      reverse: reverse,
      itemBuilder: (_, index) {
        if (header != null && index == 0) {
          return header!;
        }
        final realIndex = index - offset;
        return builder(source.item(realIndex));
      },
      separatorBuilder: (_, __) => separator ?? Container(),
      itemCount: rx.length + offset,
    );
  }

  Widget _empty(
    List<T> list,
    Widget? empty,
  ) =>
      FutureBuilder<bool>(
        future: Future.delayed(Duration(seconds: 1)).then((value) => true),
        builder: (_, s) =>
            s.hasData && list.isEmpty ? empty ?? Container() : Container(),
      );
}
